/**
 * A basic controller for scenes.
 * Contains functions virtually every scene should need.
 * It is recommended to always inherit base scene controllers from the SceneController.
 * Uses the "autosave" and "continue" tokens.
 * 
 * Capabilities:
 * -autosave
 * -continue
 * -continued
 * 
 * @param {string} id The id of the scene.
 */
function SceneController(id) {
    /*
     * Autosaves the game if necessary.
     */
    this.autosave = function () {
        global.kobold.history.doIfNotSet(id, "autosave", true, function () {
            global.saveManager.autoSave();
        });
    };

    /**
     * Signals the scene has been continued.
     */
    this.continue = function () {
        global.kobold.history.setKey(id, "continue", true);
    };

    /**
     * Returns whether or not the scene has continued.
     * @returns {boolean} True if the scene has continued, false otherwise.
     */
    this.continued = function () {
        return global.kobold.history.has(id, "continue");
    };
}

