/**
 * This class contains functions specific to the Stats tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function StatsTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "statstab";
    var title = "Stats";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds the comment for the Agility attribute.
     * @returns {string} The comment.
     */
    function buildAgilityComment() {
        var statValue = global.kobold.stats.get(StatsEnum.AGILITY);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Can't reach own toes.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">A little stiff.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Nimble.</div>`;
        }

        return `<div class="text nomargin">Part snake.</div>`;
    }

    /**
     * Builds the header for the Agility attribute.
     * @returns {String} The header.
     */
    function buildAgilityHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.AGILITY);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Agility attribute.
     * @returns {String} The number.
     */
    function buildAgilityNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.AGILITY);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Charisma attribute.
     * @returns {string} The comment.
     */
    function buildCharismaComment() {
        var statValue = global.kobold.stats.get(StatsEnum.CHARISMA);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Pariah.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">Unlikable.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Friendly.</div>`;
        }

        return `<div class="text nomargin">Inspirational.</div>`;
    }

    /**
     * Builds the header for the Charisma attribute.
     * @returns {String} The header.
     */
    function buildCharismaHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.CHARISMA);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Charisma attribute.
     * @returns {String} The number.
     */
    function buildCharismaNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.CHARISMA);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Endurance attribute.
     * @returns {string} The comment.
     */
    function buildEnduranceComment() {
        var statValue = global.kobold.stats.get(StatsEnum.ENDURANCE);
        if (statValue < 3) {
            return `<div class="text warningtext">Wimp.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text sneakytext">Soft-skinned.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text">Resillient.</div>`;
        }

        return `<div class="text">Tough.</div>`;
    }

    /**
     * Builds the header for the Endurance attribute.
     * @returns {String} The header.
     */
    function buildEnduranceHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.ENDURANCE);
        return `<div class="text">${statName}</div>`;
    }

    /**
     * Builds the number for the Endurance attribute.
     * @returns {String} The number.
     */
    function buildEnduranceNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.ENDURANCE);
        return `<div class="text">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Intelligence attribute.
     * @returns {string} The comment.
     */
    function buildIntelligenceComment() {
        var statValue = global.kobold.stats.get(StatsEnum.INTELLIGENCE);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Dumbass.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">Dim-witted.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Smart.</div>`;
        }

        return `<div class="text nomargin">Genius.</div>`;
    }

    /**
     * Builds the header for the Intelligence attribute.
     * @returns {String} The header.
     */
    function buildIntelligenceHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.INTELLIGENCE);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Intelligence attribute.
     * @returns {String} The number.
     */
    function buildIntelligenceNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.INTELLIGENCE);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Luck attribute.
     * @returns {string} The comment.
     */
    function buildLuckComment() {
        var statValue = global.kobold.stats.get(StatsEnum.LUCK);
        if (statValue < 3) {
            return `<div class="text warningtext">Doomed.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text sneakytext">Ill-fated.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text">Lucky.</div>`;
        }

        return `<div class="text">Blessed.</div>`;
    }

    /**
     * Builds the header for the Luck attribute.
     * @returns {String} The header.
     */
    function buildLuckHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.LUCK);
        return `<div class="text">${statName}</div>`;
    }

    /**
     * Builds the number for the Luck attribute.
     * @returns {String} The number.
     */
    function buildLuckNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.LUCK);
        return `<div class="text">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Strength attribute.
     * @returns {string} The comment.
     */
    function buildStrengthComment() {
        var statValue = global.kobold.stats.get(StatsEnum.STRENGTH);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Noodle arms.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">Weak.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Strong.</div>`;
        }

        return `<div class="text nomargin">Koboldcules.</div>`;
    }

    /**
     * Builds the header for the Strength attribute.
     * @returns {String} The header.
     */
    function buildStrengthHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.STRENGTH);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Strength attribute.
     * @returns {String} The number.
     */
    function buildStrengthNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.STRENGTH);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the headers for the left hand side of the attributes section.
     * @returns {string} The headers for the left hand side of the attributes section.
     */
    function buildLeftStatHeaders() {
        var headers = `<div class="statname">`;
        headers += buildAgilityHeader();
        headers += buildStrengthHeader();
        headers += buildEnduranceHeader();
        return headers + `</div>`;
    }

    /**
     * Builds the numbers for the left hand side of the attributes section.
     * @returns {string} The numbers for the left hand side of the attributes section.
     */
    function buildLeftStatNumbers() {
        var numbers = `<div class="statnumber">`;
        numbers += buildAgilityNumber();
        numbers += buildStrengthNumber();
        numbers += buildEnduranceNumber();
        return numbers + `</div>`;
    }

    /**
     * Builds the comments for the left hand side of the attributes section.
     * @returns {string} The comments for the left hand side of the attributes section.
     */
    function buildLeftStatComments() {
        var comments = `<div class="statcomment">`;
        comments += buildAgilityComment();
        comments += buildStrengthComment();
        comments += buildEnduranceComment();
        return comments + `</div>`;
    }

    /**
     * Builds the headers for the right hand side of the attributes section.
     * @returns {string} The headers for the right hand side of the attributes section.
     */
    function buildRightStatHeaders() {
        var headers = `<div class="statname">`;
        headers += buildIntelligenceHeader();
        headers += buildCharismaHeader();
        headers += buildLuckHeader();
        return headers + `</div>`;
    }

    /**
     * Builds the numbers for the right hand side of the attributes section.
     * @returns {string} The numbers for the right hand side of the attributes section.
     */
    function buildRightStatNumbers() {
        var numbers = `<div class="statnumber">`;
        numbers += buildIntelligenceNumber();
        numbers += buildCharismaNumber();
        numbers += buildLuckNumber();
        return numbers + `</div>`;
    }

    /**
     * Builds the comments for the right hand side of the attributes section.
     * @returns {string} The comments for the right hand side of the attributes section.
     */
    function buildRightStatComments() {
        var comments = `<div class="statcomment">`;
        comments += buildIntelligenceComment();
        comments += buildCharismaComment();
        comments += buildLuckComment();
        return comments + `</div>`;
    }

    /**
     * Builds and returns the stats half of the stats tab.
     * @returns {string} The stats half of the stats tab as a HTML string.
     */
    function buildStats() {
        var subtitle = `<h2 class="subtitle">Attributes</h2>`;
        var leftHalf = `<div class="half">`;
        leftHalf += buildLeftStatHeaders();
        leftHalf += buildLeftStatNumbers();
        leftHalf += buildLeftStatComments();
        leftHalf += `</div>`;
        var rightHalf = `<div class="half">`;
        rightHalf += buildRightStatHeaders();
        rightHalf += buildRightStatNumbers();
        rightHalf += buildRightStatComments();
        rightHalf += `</div>`;
        return subtitle + leftHalf + rightHalf;
    }

    /**
     * Builds the gender section of the body part of the stats tab.
     * @returns {string} The gender section of the body part of the stats tab.
     */
    function buildGender() {
        var gender = global.kobold.body.gender;
        var toReturn = `<div class="text nomargin">`;

        if (gender == GenderEnum.NONE) {
            toReturn += "You are amorphic.";
        }

        if (gender == GenderEnum.MALE) {
            toReturn += "You are a boy.";
        }

        if (gender == GenderEnum.FEMALE) {
            toReturn += "You are a girl.";
        }

        if (gender == GenderEnum.SHEMALE) {
            toReturn += "You are a girl with boy parts.";
        }

        if (gender == GenderEnum.HERMAPHRODITE) {
            toReturn += "You are both a boy and a girl.";
        }

        if (gender == GenderEnum.CUNTBOY) {
            toReturn += "You are a boy with girl parts.";
        }

        if (gender == GenderEnum.MALEHERM) {
            toReturn += "You are a boy with a cunt, but not a cuntboy.";
        }

        toReturn += ` Your name is <span class="bold">${global.kobold.name}</span>.`;

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the breasts section of the body part of the stats tab.
     * @returns {string} The breasts section of the body part of the stats tab.
     */
    function buildBreasts() {
        var bodyPart = global.kobold.body.hasBreasts();
        var lactation = global.kobold.body.lactating;
        var toReturn = `<div class="text nomargin">`;

        if (!bodyPart) {
            toReturn += `Your chest is smooth and flat.`;
        }

        if (bodyPart == SizeEnum.SMALLEST) {
            toReturn += `Your breasts are barely large enough to notice.`;
            if (lactation) {
                toReturn += " You produce only the tiniest bit of milk.";
            }
        }

        if (bodyPart == SizeEnum.SMALL) {
            toReturn += `Two small tits adorn your chest.`;
            if (lactation) {
                toReturn += " Your nipples occasionally dampen from the milk produced within.";
            }
        }

        if (bodyPart == SizeEnum.AVERAGE) {
            toReturn += `You have a pair of average-sized breasts. They look nice on you.`;
            if (lactation) {
                toReturn += " Small droplets of milk sometimes form on your nipples, rolling down your scaly body or dampening your clothes.";
            }
        }

        if (bodyPart == SizeEnum.LARGE) {
            toReturn += `Your two sizable boobs attract plenty of attention.`;
            if (lactation) {
                toReturn += " Your breasts squirt milk when squeezed.";
            }
        }

        if (bodyPart == SizeEnum.LARGER) {
            toReturn += `Your pair of oversized tits are weighing you down.`;
            if (lactation) {
                toReturn += " A slow, yet constant stream of milk leaks from your chest.";
            }
        }

        if (bodyPart == SizeEnum.HUGE) {
            toReturn += `<span class="warningtext">With <span class="bold">udders</span> this big, it's a miracle you can still move around.</span>`;
            if (lactation) {
                toReturn += " Your huge tits need to be milked hourly to stop them from constantly muddying the ground.";
            }
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the dick section of the body part of the stats tab.
     * @returns {string} The dick section of the body part of the stats tab.
     */
    function buildDick() {
        var bodyPart = global.kobold.body.hasDick();
        var toReturn = `<div class="text nomargin">`;

        if (!bodyPart) {
            return "";
        }

        if (bodyPart == SizeEnum.SMALLEST) {
            toReturn += `A tiny member resides inside your cockslit, about large enough to pleasure a virginal feral bunny.`;
        }

        if (bodyPart == SizeEnum.SMALL) {
            toReturn += `The small member resting in your cockslit is big enough to satisfy most kobolds.`;
        }

        if (bodyPart == SizeEnum.AVERAGE) {
            toReturn += `Your dick, safely concealed within your cockslit, is big enough to bring humans to orgasm.`;
        }

        if (bodyPart == SizeEnum.LARGE) {
            toReturn += `Canines and felines are typically more gifted than humans, when it comes to dicks. You measure up to them quite nicely. You could give a feral horse a run for its money.`;
        }

        if (bodyPart == SizeEnum.LARGER) {
            toReturn += `The slit between your leg is growing too small for the huge member within. Your length rivals even the most gifted of anthros. You could please <span class="bold">small dragons</span> with it.`;
        }

        if (bodyPart == SizeEnum.HUGE) {
            toReturn += `<span class="warningtext">Your oversized schlong can <span class="bold">barely</span> even fit inside of <span class="bold">fully grown dragons</span>.</span>`;
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the cunt section of the body part of the stats tab.
     * @returns {string} The cunt section of the body part of the stats tab.
     */
    function buildCunt() {
        var cunt = global.kobold.body.hasCunt();
        var dick = global.kobold.body.hasDick();
        var virgin = global.kobold.body.vaginalVirgin;
        var toReturn = `<div class="text nomargin">`;

        if (!cunt && !dick) {
            toReturn += "You have not a single slit between your legs.";
        }

        if (!cunt && dick) {
            toReturn += "Your cockslit is the only slit you have.";
        }

        if (cunt && !dick && !virgin) {
            toReturn += "While no longer a virgin, your cunt continues to allure a multitude of molesters.";
        }

        if (cunt && !dick && virgin) {
            toReturn += "The virginal cunt between your legs tempts your every assailant.";
        }

        if (cunt && dick && !virgin) {
            toReturn += "Behind your cockslit rests a second, slightly more puffy mound.";
        }

        if (cunt && dick && virgin) {
            toReturn += "Past your cockslit, your virginal twat lays ripe for the picking.";
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the ass section of the body part of the stats tab.
     * @returns {string} The ass section of the body part of the stats tab.
     */
    function buildAss() {
        var virgin = global.kobold.body.analVirgin;
        var toReturn = `<div class="text">`;

        if (!virgin) {
            toReturn += "Your tailhole is positioned in between your buttcheeks. Where else would it be?";
        }

        if (virgin) {
            toReturn += "Your tail covers your virginal butthole.";
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the Body part of the stats tab.
     * @returns {string} The Body part of the stats tab as a HTML string.
     */
    function buildBody() {
        var body = `<h2 class="subtitle">Body</h2>`;
        body += buildGender();
        body += buildBreasts();
        body += buildDick();
        body += buildCunt();
        body += buildAss();
        return body;
    }

    /**
     * Builds and returns the oral virgin status part of the stats tab.
     * @returns {string} The oral virgin status part of the stats tab.
     */
    function buildOralVirginStatus() {
        if (global.kobold.body.oralVirgin) {
            return `<div class="text nomargin">You haven't been made to suck anyone off or eat anyone out yet.</div>`;
        }
        return `<div class="text nomargin">Your muzzle has been used. You can still taste whatever you were made to service.</div>`;
    }

    /**
     * Builds and returns the anal virgin status part of the stats tab.
     * @returns {string} The anal virgin status part of the stats tab.
     */
    function buildAnalVirginStatus() {
        if (global.kobold.body.analVirgin) {
            return `<div class="text nomargin">Your ass has never been fucked. You intend to keep it that way.</div>`;
        }
        return `<div class="text nomargin">Your tailhole remains a little sore from the intense pounding it received.</div>`;
    }

    /**
     * Builds and returns the cockslit virgin status part of the stats tab.
     * @returns {string} The cockslit virgin status part of the stats tab.
     */
    function buildCockslitVirginStatus() {
        if (!global.kobold.body.hasDick()) {
            return ``;
        }
        if (global.kobold.body.cockslitVirgin) {
            return `<div class="text nomargin">Your cockslit has never been fucked before. Thank goodness!</div>`;
        }
        return `<div class="text nomargin">The tight, warm confines of your cockslit have been defiled.</div>`;
    }

    /**
     * Builds and returns the cock virgin status part of the stats tab.
     * @returns {string} The cock virgin status part of the stats tab.
     */
    function buildCockVirginStatus() {
        if (!global.kobold.body.hasDick()) {
            return ``;
        }
        if (global.kobold.body.dickVirgin) {
            return `<div class="text nomargin">You've never fucked anyone with your member.</div>`;
        }
        return `<div class="text nomargin">You finally got laid!</div>`;
    }

    /**
     * Builds and returns the vaginal virgin status part of the stats tab.
     * @returns {string} The vaginal virgin status part of the stats tab.
     */
    function buildVaginalVirginStatus() {
        if (!global.kobold.body.hasCunt()) {
            return ``;
        }
        if (global.kobold.body.vaginalVirgin) {
            return `<div class="text nomargin">Your puffy mound remains untouched.</div>`;
        }
        return `<div class="text nomargin">You have been robbed of your maidenhood.</div>`;
    }

    /**
     * Builds and returns a summary of virgin status.
     * @returns {string} A summary of virgin status.
     */
    function buildVirginSummary() {
        if (global.kobold.body.virgin()) {
            return `<div class="text nomargin">You remain a pure virgin.</div>`;
        }
        return `<div class="text nomargin">You are no longer a virgin.</div>`;
    }

    /**
     * Builds and returns the virgin status part of the stats tab.
     * @returns {string} The virgin status part of the stats tab as a HTML string.
     */
    function buildVirginStatus() {
        var virginStatus = `<h2 class="subtitle">Virginity</h2>`;
        virginStatus += buildOralVirginStatus();
        virginStatus += buildAnalVirginStatus();
        virginStatus += buildCockVirginStatus();
        virginStatus += buildCockslitVirginStatus();
        virginStatus += buildVaginalVirginStatus();
        virginStatus += buildVirginSummary();
        return virginStatus;
    }

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = gen.buildTabHeader(title);
        tab += buildStats();
        tab += buildBody();
        tab += buildVirginStatus();
        tab += gen.buildTabFooter();
        return tab;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        global.ui.imageBackground.hide();
        var tab = buildTab();
        global.ui.loadToMain(tab);
    };
}
