/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseSteal(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var toolshedId = arguments.callee.name + "ButtonOne";
    var insideId = arguments.callee.name + "ButtonTwo";

    var toToolshed = new FarmerHouseShed(ctrl);
    var toInside = new FarmerHouseInside(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You press yourself against the wall, hiding behind a tall cabinet to ensure you won't be spotted while you think of a plan.
                            The woman softly hums to herself while she goes about her business, making sandwiches for dinner.
                            While the hallway you are in somewhat separates the two of you, the only way further into the house is via the kitchen,
                            and you doubt you can sneak by the human without her noticing.
                        [/p]
                        [p]
                            You struggle to come up with an alternate route that won't lead to your detection.
                            You [i]could[/i] leave out the back and go through the front door, or maybe try a different house,
                            but you'd essentially be trading one roll of the dice for another.
                            Who's to say there won't be a guard dog or another housewife or an inquisitive child next door?
                            Heck, you got [b]lucky[/b] this time around.
                            The door was unlocked, and you managed to enter without making much noise.
                            Besides, if the woman is making dinner, you doubt it'll be long before the men of the village return.
                        [/p]
                        [p]
                            A thought bubbles up in your brain.
                            The shed outside.
                            You [i]could[/i] go take a look in there.
                            It'd be [b]safer[/b] than staying in the house,
                            and you might find something useful, although probably nothing [i]too[/i] valuable.
                            But then again, this place doesn't look too promising either.
                            [b]If[/b] you can get anything at all from it without the woman in the kitchen spotting you.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildToolshedChoice() {
        var disabled = ctrl.steal.has() ? "disabled" : "";
        var isSelected = ctrl.steal.get() === FarmerHouseStealChoice.SHED;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${toolshedId}" ${disabled}>Go to the shed</button>`;

        var tooltip = global.parser.parse(`At least the toolshed outside won't have a housewife running around in it.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 195px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildInsideChoice() {
        var disabled = ctrl.steal.has() ? "disabled" : "";
        var isSelected = ctrl.steal.get() === FarmerHouseStealChoice.INSIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${insideId}" ${disabled}>Stay indoors</button>`;

        var tooltip = global.parser.parse(`Profit trumps risk. Find a way to steal things without being noticed.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 150px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildToolshedChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildInsideChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + toolshedId), $("#" + insideId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerToolshedListener() {
        $("#" + toolshedId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.steal.set(FarmerHouseStealChoice.SHED);
            loadNextPart(toToolshed);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerInsideListener() {
        $("#" + insideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.steal.set(FarmerHouseStealChoice.INSIDE);
            loadNextPart(toInside);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.arrival.get() !== FarmerHouseArrivalChoice.STEAL) {
            return "";
        }
        
        global.kobold.history.shared.farmerhouse.stole = true;
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toToolshed.build();
        part += toInside.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.arrival.get() !== FarmerHouseArrivalChoice.STEAL) {
            return;
        }
        
        toToolshed.registerListeners();
        toInside.registerListeners();

        if (ctrl.steal.has()) {
            return;
        }

        registerToolshedListener();
        registerInsideListener();
    };
}

var FarmerHouseStealChoice = {
    SHED: 1,
    INSIDE: 2
};

