/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseDrawer(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var nextId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseCabinetRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You can't open the cupboard, and you can't reach the top shelf.
                            That leaves the drawer.
                            You grab the metallic handle and tug on it, half-expecting the tray to be jammed much like the bottom compartiment is.
                            To your surprise, the drawer opens rather smoothly.
                            Overestimating the amount of force needed, you almost pull it out completely, your yank luckily stopping just short of disaster.
                        [/p]
                        [p]
                            The drawer seems to be mostly filled with worthless stuff.
                            Letters, a ball of yarn, an old, moldy cookie, some religious book you don't recognize and a spare broom head.
                            Disappointed, you begin to rummage through the stuff.
                            Surely there's something better underneath all of this [b]crap[/b], right?
                        [/p]
                        [p]
                            Nothing.
                            You start to get annoyed.
                            You toss the letters to the floor.
                            You put the well-worn book on the cabinet.
                            You throw the ball of yarn across the hallway in aggravation.
                            And then, hiding in the corner of the drawer, beyond the now-discarded thread, your prize is revealed to you:
                            a single ${global.kobold.inventory.createItemWithTooltip(new CopperRing())}.
                        [/p]
                        [p]
                            It probably slipped off of someone's finger, or maybe it was stored in there for safekeeping.
                            Regardless, you pocket it as your just due for having to stand through the harrowing sensation of almost being caught.
                            It isn't nearly as much as you'd hoped to gain from this venture, but it's more than nothing.
                            Honestly, you shouldn't have expected anything more valuable coming from a farmer's home.
                            You're glad you got at least [b]something[/b] out of this ordeal.
                        [/p]
                        [p]
                            You don't bother cleaning up after yourself.
                            Honestly, you're doing these dumb, disorganized humans a favor by emptying out their drawer full of trash.
                            Maybe when they pick up the stuff you threw to the floor, they'll store it in a proper fashion.
                            [i]And get rid of that disgusting cookie in the process.[/i]
                            Still somewhat frustrated, you silently slip out through the back door from whence you came.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.cabinetSteal.continued() ? "disabled" : "";
        var isSelected = ctrl.cabinetSteal.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${nextId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + nextId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + nextId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.cabinetSteal.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.DRAWER) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.cabinetSteal.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.DRAWER) {
            return;
        }

        next.registerListeners();

        if (ctrl.cabinetSteal.continued()) {
            return;
        }

        registerNextListener();
    };
}
