/**
 * Hammock section of the home scene.
 * @param {HomeController} controller The home controller.
 */
function HomeHammock(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueHamletId = `toHamletContinueButton`;
    var continueVaranarId = `toVaranarContinueButton`;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You decide you'd rather not risk sleeping inside of your home in case the Rat's men return.
                            Seeing as you don't think you'd get much sleep at all on the cold, hard ground, 
                            you choose to spend your night on the hammock, 
                            an excellent compromise between comfort and the risk of getting caught by [i]whoever[/i] is looking for you.
                        [/p]
                        [p]
                            Even [i]if[/i] someone were to come [i]back[/i] to your shack during the night, 
                            you reason there's no way they'd come looking for you in the forest itself.
                            While you'll be suspended above any sort of concealing shrubbery, 
                            your slightly more remote location as well as the darkness of night are sure to hide you from any prying eyes.
                        [/p]
                        [p]
                            You march into the woods beyond your hovel.
                            Even you, having a good idea of where your hammock [i]should[/i] be, have a hard time actually [b]finding[/b] it in the middle of night.
                            Only after a good five minutes of determined searching do you finally come across your chosen nighttime solution.
                            Of course, with someone sleeping in it, it's bound to be a little more visible than when empty, but oh well.
                            You've made your bed, and now you're going to lie in it.
                        [/p]
                        [p]
                            You crawl into the hammock, and after a bit of fumbling, manage to reach a comfortable resting position.
                            The only thing you didn't consider was the temperature.
                            It's pretty chilly outside, and you have nothing to cover yourself with.
                            In an attempt to conserve precious body heat, you curl up into a ball.
                            Shuddering from the cold, you slowly drift to sleep.
                        [/p]
                        [p]
                            While the near-freezing temperature rouses you from your slumber more than once,
                            it does so for only a few minutes at a time before you drift back into unconsciousness.
                            You manage to get a decent amount of oft-interrupted rest in.
                        [/p]
                        [p]
                            As the sun begins to rise, you wake up properly, chilled to the very bone.
                            Shivering uncontrollably, you slip out of your hammock.
                            You breathe some warm air onto your ice-cold scaly hands and rub them together to create friction.
                        [/p]
                        [p]
                            While you wish you'd be able to wake up nice and slow,
                            your frigid surroundings spur you to immediately begin to move about in order to generate some much-needed body heat.
                            You rub your eyes, stretch your legs, and walk back to your shed.
                        [/p]
                        [p]
                            It looks to be in exactly the same condition you left it yesterday.
                            You're starting to regret not sleeping indoors.
                            But alas, what is done, is done.
                        [/p]
                        [p]
                            You continue to warm up your body by rubbing your legs and your arms with your hands.
                            With the sun gradually warming your surroundings, 
                            you ultimately manage to reach the point where you're no longer [b]too[/b] cold to think.
                        [/p]
                        [p]`;

        var gold = ctrl.gold();
        if (!gold) {
            toReturn += `   You are completely [b]broke[/b]. 
                            You have no money whatsoever to pay your overdue debts with.
                            Desperate times call for desperate measures.`;
        } else if (gold > 150) {
            toReturn += `   You have quite a bit of money, but sadly it's still not enough to pay your overdue debt.`;
        } else if (gold > 203) {
            toReturn += `   You have [b]a lot[/b] of money, more than you gained from just robbing the caravan, even.
                            You're [b]very[/b] close to being able to pay off your debt.
                            You just need a [i]teensy[/i] bit more.
                            But you aren't going to get it from these woods.`;
        }

        toReturn += `       You decide to head out, to seek your fortune in the great city of Varanar.
                            Maybe you'll stop at the small hamlet outside of the city walls along the way.
                            If anything, you hope you'll at least sleep somewhere a little warmer tonight.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.hasContinued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueHamletId}" ${disabled ? "disabled" : ""} style="margin: auto; float: left;">`;
        choice += `To the hamlet &gt;&gt`;
        choice += `</button>`;
        choice += `<button class="button bigbutton continuebutton" id="${continueVaranarId}" ${disabled ? "disabled" : ""} style="margin: auto; float: right;">`;
        choice += `To Varanar &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }
    
    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueHamletId), $("#" + continueVaranarId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }
    
    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }
    
    /**
     * Adds a listener to the hamlet button.
     */
    function registerHamletListener() {
        $("#" + continueHamletId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.hamletSceneName, ctrl.hamletFolder);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "homeHammock";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.unsaved.getSleepChoice() !== HomeUnsavedChoice.HAMMOCK) {
            return "";
        }
        global.ui.applyTheme("home_dawn");
        var change = ctrl.unsaved.hammockChange();
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += change;
        part += buildChoice();
        part += `<span class="todo">You can only go to the hamlet.</span>`;
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.unsaved.getSleepChoice() !== HomeUnsavedChoice.HAMMOCK) {
            return;
        }

        // REGISTER CHILDREN HERE

        if (ctrl.hasContinued()) {
            return;
        }

        registerHamletListener();
    };
}
