/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlley(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var hideId = arguments.callee.name + "ButtonOne";
    var runId = arguments.callee.name + "ButtonTwo";

    var toHide = new NightTwoHide(ctrl);
    var toRun = new NightTwoAlleyRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Desperate to get away from the men, your mind forms a fickle escape plan.
                            You turn a corner, and immediately [b]bolt[/b] for the nearest alleyway.
                            Stepping into the tight, unlit corridor, you let your eyes adjust to the complete darkness for a few moments, your superb nightvision kicking in.
                            The alley you've entered is a dead end, the other side blocked off by a pretty high brick barrier.
                            A collection of barrels are unorderly arrayed against the leftmost wall, leaving barely enough room for a full-grown human to squeeze on by.
                        [/p]
                        [p]
                            This place [b]stinks[/b] like rotten fish.
                            You consider backing out and either picking a different alleyway, or just ditching the idea of hiding completely, but alas, it is already too late.
                            Judging by the noise coming from the street, the men have followed in your footsteps, and they are currently out looking for you.
                            If you remain standing in the entrance, they'll undoubtebly find you.
                            You need to make a decision, and quick.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildHideChoice() {
        var disabled = ctrl.alley.has() ? "disabled" : "";
        var isSelected = ctrl.alley.get() === NightTwoAlleyChoice.HIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${hideId}" ${disabled}>Hide</button>`;

        var tooltip = global.parser.parse(`Conceal yourself behind the barrels.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildRunChoice() {
        var disabled = ctrl.alley.has() ? "disabled" : "";
        var isSelected = ctrl.alley.get() === NightTwoAlleyChoice.RUN;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${runId}" ${disabled}>Run</button>`;

        var tooltip = global.parser.parse(`Try to parkour up and over the wall.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildRunChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildHideChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + hideId), $("#" + runId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerRunListener() {
        $("#" + runId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alley.set(NightTwoAlleyChoice.RUN);
            loadNextPart(toRun);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerHideListener() {
        $("#" + hideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alley.set(NightTwoAlleyChoice.HIDE);
            loadNextPart(toHide);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.flee.get() !== NightTwoFleeChoice.HIDE) {
            return "";
        }

        global.ui.applyTheme("varanar_alley");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toRun.build();
        part += toHide.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.flee.get() !== NightTwoFleeChoice.HIDE) {
            return;
        }

        toRun.registerListeners();
        toHide.registerListeners();

        if (ctrl.alley.has()) {
            return;
        }

        registerRunListener();
        registerHideListener();
    };
}

var NightTwoAlleyChoice = {
    HIDE: 1,
    RUN: 2
};

